

#include "DemoConfig.h"

#if defined(__APPLE__)
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif



DemoEventHandler handler;
GuiWrapper gui(&handler);

void MouseClicked(int button, int state, int x, int y) ;
void MouseMoved(int x, int y);
void KeyTyped(unsigned char key, int x, int y);
void InitializeScene();
void SetPerspective(int width, int height);
void RenderScene();
void InputKey(int key, int x, int y);

int main(int argc, char *argv[])
{

	std::string DataDir = DEMO_DATA_DIR;
	if (argc == 2) DataDir = argv[1];

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(700, 500);
  glutCreateWindow("GLUT Demo");


  // Keyboard
  glutKeyboardFunc(KeyTyped);
  glutSpecialFunc(InputKey);

  // Display and Idle
  glutDisplayFunc(RenderScene);
  glutIdleFunc(RenderScene);

  glutReshapeFunc(SetPerspective);

  // Mouse stuff
  glutMouseFunc(MouseClicked);
  glutPassiveMotionFunc(MouseMoved);
  glutMotionFunc(MouseMoved);

  gui.Init(DataDir.c_str());

  glutMainLoop();

  return(0);
}

int oldTime = 0;

void RenderScene()
{
	glClearColor(handler.red, handler.green, handler.blue, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//get elapsed milliseconds
	int time = glutGet(GLUT_ELAPSED_TIME);
	gui.Update((time-oldTime)/1000.0f);
	oldTime = time;

	if (gui.IsActive()) gui.RenderScene();
	glutSwapBuffers();
}


void SetPerspective( int width, int height)
{
  float aspectRatio = 1.33f;
  height            = height <= 0 ? 1 : height;
  aspectRatio       = (float)width/height;

  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);

  glLoadIdentity();
  gluPerspective(90.0f, aspectRatio, 1.0f, 1500.0f);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  gui.SetDimensions(width, height);
}

void MouseClicked(int button, int state, int x, int y)
{
  button  = (button == GLUT_LEFT_BUTTON ) ? 1 :
            (button == GLUT_RIGHT_BUTTON) ? 2 : 3;

	gui.MouseClicked(button, (state == GLUT_DOWN), x, y);
}

void MouseMoved(int x, int y)
{
	gui.MouseMoved(x, y);
}

void KeyTyped(unsigned char key, int x, int y)
{
  gui.AsciiKeyDown(key);

  if (key == 27)
    exit(0);
}

void InputKey(int key, int x, int y)
{
	gui.SpecialKeyDown(key);
}

