#include "GuiWrapper.h"

#include "glwx.h"

#include <string>

GuiWrapper::GuiWrapper(GUIEventListener * eventListener)
{
	handler = eventListener;
	fpsDisplay = NULL;
	dt = 0;
	elapsed = 0;
	frames = 0;
	isActive = true;
	guiFrame = new GUIFrame;
}

GuiWrapper::~GuiWrapper()
{
	TexturesManager::flushAllTextures();
}

/**
* Initializes GLWX
* @param dataPath A character string to the data directory containing the Layouts and Textures directories.
* @return true if success
*/
bool GuiWrapper::Init(const char * dataPath)
{
	Logger::initialize();

	std::string path = dataPath;
	std::string texturesPath = path + "/Textures/";
	std::string layoutsPath = path + "/Layouts/";
	std::string layout = layoutsPath + "Layout.xml";

	MediaPathManager::registerPath(texturesPath.c_str());
	MediaPathManager::registerPath(layoutsPath.c_str());
	guiFrame->GUIPanel::loadXMLSettings(layout.c_str());
	guiFrame->setGUIEventListener(handler);

	fpsDisplay = (GUILabel*)guiFrame->getWidgetByCallbackString("fpsCounter");

	Logger::flush();
	return true;
}

bool GuiWrapper::MouseClicked(int button, bool isDown, int x, int y)
{
  button  = (button == 1 ) ? MB_BUTTON1 :
            (button == 2) ? MB_BUTTON2 : MB_BUTTON3;

  MouseEvent event = MouseEvent(MB_BUTTON1, x, y, guiFrame->getHeight() - y);
  guiFrame->checkMouseEvents(event, (isDown == true) ? ME_CLICKED: ME_RELEASED);
  return event.getUsed();

}

bool GuiWrapper::MouseMoved(int x, int y)
{
  MouseEvent event = MouseEvent(MB_UNKNOWN_BUTTON, x, y, guiFrame->getHeight() - y);
  guiFrame->checkMouseEvents(event, ME_MOVED);
  return event.getUsed();
}

bool GuiWrapper::AsciiKeyDown(unsigned char key)
{
	if (key == 27) this->SetActive(!this->IsActive());
	KeyEvent event(key);
	guiFrame->checkKeyboardEvents(event, KE_PRESSED);
	return event.getUsed();
}

bool GuiWrapper::SpecialKeyDown(int key) {
	// this function could be used for key modifiers
	// like shift or control.
	return false;
}

void GuiWrapper::SetDimensions(int width, int height)
{
  guiFrame->setDimensions(float(width), float(height));
  guiFrame->forceUpdate(true);
}

void GuiWrapper::RenderScene()
{

  Enter2DMode(guiFrame->getWidth(), guiFrame->getHeight());
  	guiFrame->render(dt);
  Exit2DMode();
 
  char charBuffer[50]; // FIX THIS

  if(fpsDisplay)
  {
    sprintf(charBuffer, "Current FPS: %d", int(fps) );
    fpsDisplay->setLabelString(charBuffer);
  }

}

void GuiWrapper::Update(double dt)
{
	this->dt = dt;
	elapsed += dt;
	frames++;

	if (elapsed >= 0.1)
	{
		fps = frames/elapsed;
		frames = 0;
		elapsed = 0;
	}
}

bool GuiWrapper::IsActive()
{
	return isActive;
}

void GuiWrapper::SetActive( bool active )
{
	isActive = active;
}


void GuiWrapper::Enter2DMode(int width, int height)
{
  GLint viewport[4];

  glGetIntegerv(GL_VIEWPORT, viewport);
  int winWidth  = viewport[2];
  int winHeight = viewport[3];

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  gluOrtho2D(0, winWidth, winHeight, 0);
  glDisable(GL_DEPTH_TEST);
  glDisable(GL_LIGHTING);

}

void GuiWrapper::Exit2DMode()
{
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  glEnable(GL_BLEND);

}










