
#include "SDL.h"
#include "DemoConfig.h"


DemoEventHandler handler;
GuiWrapper gui(&handler);
unsigned int oldTime = 0;

bool InitSDL();
void RenderScene();
void ResizeWindow(int width, int height);

 

int main (int argc, char *argv[]) {

	bool done = false;
	int videoFlags;
	int SCREEN_WIDTH  = 700;
	int SCREEN_HEIGHT = 500;
	int SCREEN_BPP    = 32;

	const char APP_NAME[] = "SDL Demo";

	std::string DataDir = DEMO_DATA_DIR;
	if (argc == 2) DataDir = argv[1];

	SDL_Event event;
	SDL_Surface *surface;

	//init sdl and sdl_net
	if ( !InitSDL() ) {
		printf("Video initialization failed.\n");
		exit(0);
	}

	//call SDL_Quit if program exits
	atexit(SDL_Quit);


	//set openGL attribute
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	//set window title
	SDL_WM_SetCaption(APP_NAME, APP_NAME);

	//set video flags
	videoFlags = SDL_OPENGL | SDL_RESIZABLE;    

	// get a SDL surface
	surface = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
				videoFlags );

	//check that we got a surface
	if ( !surface ) {
		printf("Video mode set failed: %s\n", SDL_GetError( ) );
		exit(0);
	}


	//initialize gui
	if ( !gui.Init(DataDir.c_str()) ) {
		printf("Failed to initialize gui.\n");
		exit(0);
	}

	//initialize window size
	ResizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

	//--- main loop ----

	while ( !done ) {

	    //handle the events in the queue 
	    while ( SDL_PollEvent( &event ) )
		{
		    switch( event.type )
			{
			    
			case SDL_VIDEORESIZE:
				surface = SDL_SetVideoMode( event.resize.w, event.resize.h, SCREEN_BPP, videoFlags );
				if ( !surface ) {
					printf("Could not get a surface after resize: %s\n", SDL_GetError() );
					exit(0);
				}

				ResizeWindow( event.resize.w, event.resize.h );
				RenderScene();
				break;
			case SDL_KEYDOWN:
				gui.AsciiKeyDown( event.key.keysym.sym );
				break;
			case SDL_MOUSEMOTION:
				gui.MouseMoved(event.motion.x, event.motion.y); //event.motion.xrel, event.motion.yrel
				break;
			case SDL_MOUSEBUTTONDOWN:
				if (event.button.button == SDL_BUTTON_LEFT)
					gui.MouseClicked(1, true, event.button.x, event.button.y);
				else if (event.button.button == SDL_BUTTON_RIGHT)
					gui.MouseClicked(2, true, event.button.x, event.button.y);
				break;
			case SDL_MOUSEBUTTONUP:
				if (event.button.button == SDL_BUTTON_LEFT)
					gui.MouseClicked(1, false, event.button.x, event.button.y);
				else if (event.button.button == SDL_BUTTON_RIGHT)
					gui.MouseClicked(2, false, event.button.x, event.button.y);
				break;
			case SDL_QUIT:
				done = true;
				break;
			default:
				break;
			}
		}

		if (!done) {

			//render
			RenderScene();
		}
	}

	SDL_Quit();

	return 0;
}

bool InitSDL() {

	// initialize SDL
	if(SDL_Init( SDL_INIT_VIDEO ) < 0)
	{
		printf("SDL_Init: %s\n",SDL_GetError());
		return false;
	}

	//turn on keyboard key repeat
	if ( ( SDL_EnableKeyRepeat( 500, SDL_DEFAULT_REPEAT_INTERVAL ) ) ) {
		printf("Setting keyboard repeat failed: %s\n", SDL_GetError( ) );
		return false;
	}

	return true;
}

void ResizeWindow(int width, int height) {

  float aspectRatio = 1.33f;
  height            = height <= 0 ? 1 : height;
  aspectRatio       = (float)width/height;

  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);

  glLoadIdentity();
  gluPerspective(90.0f, aspectRatio, 1.0f, 1500.0f);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  gui.SetDimensions(width, height);

}

void RenderScene()
{
	glClearColor(handler.red, handler.green, handler.blue, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//get elapsed milliseconds
	int time = SDL_GetTicks();
	gui.Update((time-oldTime)/1000.0f);
	oldTime = time;

	if (gui.IsActive()) gui.RenderScene();
	SDL_GL_SwapBuffers( );

}





