

#include "LatticeConfig.h"
#include "lattice.h"

#include "EventHandler.h"
#include "GuiWrapper.h"



#if defined(__APPLE__)
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif


Lattice lattice;
EventHandler handler(&lattice);
GuiWrapper gui(&handler);

void MouseClicked(int button, int state, int x, int y) ;
void MouseMoved(int x, int y);
void KeyTyped(unsigned char key, int x, int y);
void InitializeScene();
void SetPerspective(int width, int height);
void RenderScene();
void InputKey(int key, int x, int y);

int main(int argc, char *argv[])
{

  std::string DataDir = LATTICE_DATA_DIR;

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(700, 500);
  glutCreateWindow("Lattice Demo");


  // Keyboard
  glutKeyboardFunc(KeyTyped);
  glutSpecialFunc(InputKey);

  // Display and Idle
  glutDisplayFunc(RenderScene);
  glutIdleFunc(RenderScene);

  glutReshapeFunc(SetPerspective);

  // Mouse stuff
  glutMouseFunc(MouseClicked);
  glutPassiveMotionFunc(MouseMoved);
  glutMotionFunc(MouseMoved);

  gui.Init(DataDir.c_str());

  lattice.hack_init(700,500);

  glutMainLoop();

  return(0);
}

int oldTime = 0;

void RenderScene()
{
	glClearColor(handler.red, handler.green, handler.blue, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	//get elapsed milliseconds
	int time = glutGet(GLUT_ELAPSED_TIME);
	double dt = (time-oldTime)/1000.0f;


	//save attributes
	glPushAttrib(GL_ALL_ATTRIB_BITS);

		lattice.hack_draw(dt);

		// save currently loaded texture
		int savedTexture = 0;
		glGetIntegerv(GL_TEXTURE_BINDING_2D,(GLint*)&savedTexture);

	//pop attributes
	glPopAttrib();

	gui.Update(dt);

	glPushAttrib(GL_ALL_ATTRIB_BITS);

		glDisable (GL_COLOR_MATERIAL);
		glEnable(GL_BLEND);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glColor4f(1,1,1,1);
		glDisable(GL_TEXTURE_2D);

		if (gui.IsActive()) gui.RenderScene();

	glPopAttrib();
	glBindTexture (GL_TEXTURE_2D, savedTexture);

	oldTime = time;

	glutSwapBuffers();
}


void SetPerspective( int width, int height)
{
  gui.SetDimensions(width, height);
  lattice.hack_reshape(width, height);
}

void MouseClicked(int button, int state, int x, int y)
{
  button  = (button == GLUT_LEFT_BUTTON ) ? 1 :
            (button == GLUT_RIGHT_BUTTON) ? 2 : 3;

	gui.MouseClicked(button, (state == GLUT_DOWN), x, y);
}

void MouseMoved(int x, int y)
{
	gui.MouseMoved(x, y);
}

void KeyTyped(unsigned char key, int x, int y)
{
  gui.AsciiKeyDown(key);
}

void InputKey(int key, int x, int y)
{
	gui.SpecialKeyDown(key);
}

