

#include "LightsConfig.h"
#include "Lights.h"

#include "EventHandler.h"
#include "GuiWrapper.h"



#if defined(__APPLE__)
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif


Lights lights;
EventHandler handler(&lights);
GuiWrapper gui(&handler, &lights);

void MouseClicked(int button, int state, int x, int y) ;
void MouseMoved(int x, int y);
void KeyTyped(unsigned char key, int x, int y);
void InitializeScene();
void SetPerspective(int width, int height);
void RenderScene();
void InputKey(int key, int x, int y);

int main(int argc, char *argv[])
{

  std::string DataDir = LIGHTS_DATA_DIR;

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(800, 600);
  glutCreateWindow("Lights Demo");


  // Keyboard
  glutKeyboardFunc(KeyTyped);
  glutSpecialFunc(InputKey);

  // Display and Idle
  glutDisplayFunc(RenderScene);
  glutIdleFunc(RenderScene);

  glutReshapeFunc(SetPerspective);

  // Mouse stuff
  glutMouseFunc(MouseClicked);
  glutPassiveMotionFunc(MouseMoved);
  glutMotionFunc(MouseMoved);

  gui.Init(DataDir.c_str());

  // Init lights demo
  lights.Init();

  glutMainLoop();

  return(0);
}

int oldTime = 0;

void RenderScene()
{

	//get elapsed milliseconds
	int time = glutGet(GLUT_ELAPSED_TIME);
	double dt = (time-oldTime)/1000.0f;


	//save attributes
	glPushAttrib(GL_ALL_ATTRIB_BITS);

		lights.Draw();

	//pop attributes
	glPopAttrib();

	gui.Update(dt);

	glPushAttrib(GL_ALL_ATTRIB_BITS);

		glEnable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);

		if (gui.IsActive()) gui.RenderScene();

	glPopAttrib();

	oldTime = time;

	glutSwapBuffers();
}


void SetPerspective( int width, int height)
{

  float aspectRatio = 1.33f;
  height            = height <= 0 ? 1 : height;
  aspectRatio       = (float)width/height;

  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);

  glLoadIdentity();
  gluPerspective( 45.0f, aspectRatio, 0.1f, 100.0f );

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  gui.SetDimensions(width, height);
}

void MouseClicked(int button, int state, int x, int y)
{
  button  = (button == GLUT_LEFT_BUTTON ) ? 1 :
            (button == GLUT_RIGHT_BUTTON) ? 2 : 3;

	if (!gui.MouseClicked(button, (state == GLUT_DOWN), x, y)) {
		if (state == GLUT_DOWN) lights.MouseDown();
		else lights.MouseUp();
	}
}

void MouseMoved(int x, int y)
{
	static int oldX = x;
	static int oldY = y;

	gui.MouseMoved(x, y);
	lights.MouseMove(x-oldX, y-oldY);

	oldX = x;
	oldY = y;
}

void KeyTyped(unsigned char key, int x, int y)
{
  gui.AsciiKeyDown(key);
}

void InputKey(int key, int x, int y)
{
	gui.SpecialKeyDown(key);
}

